import { createElementSize } from "@solid-primitives/resize-observer";
import {
  JSX,
  Show,
  children,
  createRenderEffect,
  createSignal,
  splitProps,
  type Component,
  type ParentProps,
} from "solid-js";
import "./Scaffold.css";

type ScaffoldProps = ParentProps<
  {
    topbar?: JSX.Element;
    fab?: JSX.Element;
    bottom?: JSX.Element;
  } & JSX.HTMLElementTags["div"]
>;

/**
 * The passthrough props are passed to the content container.
 */
const Scaffold: Component<ScaffoldProps> = (oprops) => {
  const [props, rest] = splitProps(oprops, [
    "topbar",
    "fab",
    "bottom",
    "children",
    "ref",
    "class",
  ]);
  const [topbarElement, setTopbarElement] = createSignal<HTMLElement>();

  const topbarSize = createElementSize(topbarElement);

  const topbar = children(() => props.topbar)
  const fab = children(() => props.fab)
  const bottom = children(() => props.bottom)

  return (
    <div
      class={`Scaffold ${props.class || ""}`}
      ref={(e) => {
        createRenderEffect(() => {
          e.style.setProperty(
            "--scaffold-topbar-height",
            (topbarSize.height?.toString() ?? 0) + "px",
          );
        });

        if (props.ref) {
          (props.ref as (val: typeof e) => void)(e);
        }
      }}
      {...rest}
    >
      <Show when={topbar()}>
        <div class="topbar" ref={setTopbarElement} role="presentation">
          {topbar()}
        </div>
      </Show>
      <Show when={fab()}>
        <div class="fab-dock" role="presentation">
          {fab()}
        </div>
      </Show>

      {props.children}

      <Show when={bottom()}>
        <div class="bottom-dock" role="presentation">
          {bottom()}
        </div>
      </Show>
    </div>
  );
};

export default Scaffold;
